# -*- coding: utf-8 -*-


def redis_message_template() -> str:
    """ """

    result = {}
    result.update({"params": {}})
    result.update({"method": "subscription"})
    result["params"].update({"data": None})
    result["params"].update({"channel": None})
    result["params"].update({"stream": None})

    return result


def trade_template() -> str:
    """
    combining result from websocket (user changes/trade)
        and rest API (get_transaction_log)

        instrument_name: str=None,
        amount: int=None,
        price: float=None,
        side: str=None,
        direction: str=None,
        position: int=None,
        currency: str=None,
        timestamp: int=None,
        trade_id: str=None,
        order_id: str=None,
        user_seq: int=None

    label is not registered since it needed as identification for no label
    """
    return dict(
        instrument_name=None,
        amount=None,
        price=None,
        side=None,
        direction=None,
        position=None,
        currency=None,
        timestamp=None,
        trade_id=None,
        order_id=None,
        user_seq=None,
        label=None,
    )


def get_custom_label(transaction: dict) -> str:

    try:

        side = transaction["direction"]

    except:

        if "sell" in transaction["side"]:
            side = "sell"

        if "buy" in transaction["side"]:
            side = "buy"

    side_label = "Short" if side == "sell" else "Long"

    try:
        last_update = transaction["timestamp"]
    except:
        try:
            last_update = transaction["last_update_timestamp"]
        except:
            last_update = transaction["creation_timestamp"]

    return f"custom{side_label.title()}-open-{last_update}"


def get_custom_label_oto(transaction: dict) -> dict:

    side = transaction["direction"]
    side_label = "Short" if side == "sell" else "Long"

    try:
        last_update = transaction["timestamp"]
    except:
        try:
            last_update = transaction["last_update_timestamp"]
        except:
            last_update = transaction["creation_timestamp"]

    return dict(
        open=(f"custom{side_label.title()}-open-{last_update}"),
        closed=(f"custom{side_label.title()}-closed-{last_update}"),
    )



# -*- coding: utf-8 -*-
"""_summary_"""
# built ins
import asyncio

# installed
from loguru import logger as log

# user defined formula
from streaming_helper.db_management import sqlite_management as db_mgt
from streaming_helper.db_management.sqlite_management import (
    executing_query_based_on_currency_or_instrument_and_strategy as get_query,
)
from streaming_helper.strategies.deribit.basic_strategy import get_label_integer
from streaming_helper.utilities import (
    string_modification as str_mod,
    error_handling,
)


def get_label_main(
    result: list,
    strategy_label: str,
) -> list:
    """ """

    return [
        o
        for o in result
        if str_mod.parsing_label(strategy_label)["main"]
        == str_mod.parsing_label(o["label"])["main"]
    ]


async def get_unrecorded_trade_id(instrument_name: str) -> dict:

    currency = str_mod.extract_currency_from_text(instrument_name)

    column_list: str = "data", "trade_id"

    from_sqlite_open = await get_query(
        "my_trades_all_json", instrument_name, "all", "all", column_list
    )

    from_sqlite_closed = await get_query(
        "my_trades_closed_json", instrument_name, "all", "all", column_list
    )

    # column_list: str="order_id", "trade_id","label","amount","id","data",
    from_sqlite_all = await get_query(
        f"my_trades_all_{currency.lower()}_json",
        instrument_name,
        "all",
        "all",
        column_list,
        # 40,
        # "id"
    )

    from_sqlite_closed_trade_id = [o["trade_id"] for o in from_sqlite_closed]

    from_sqlite_open_trade_id = [o["trade_id"] for o in from_sqlite_open]

    from_exchange_trade_id = [o["trade_id"] for o in from_sqlite_all]

    combined_trade_closed_open = from_sqlite_open_trade_id + from_sqlite_closed_trade_id

    unrecorded_trade_id = str_mod.get_unique_elements(
        from_exchange_trade_id, combined_trade_closed_open
    )

    log.debug(f"unrecorded_trade_id {unrecorded_trade_id}")

    return (
        []
        if not from_sqlite_all
        else [
            o["data"] for o in from_sqlite_all if o["trade_id"] in unrecorded_trade_id
        ]
    )


def get_custom_label(transaction: list) -> str:

    side = transaction["direction"]
    side_label = "Short" if side == "sell" else "Long"

    try:
        last_update = transaction["timestamp"]
    except:
        try:
            last_update = transaction["last_update_timestamp"]
        except:
            last_update = transaction["creation_timestamp"]

    return f"custom{side_label.title()}-open-{last_update}"


def get_unrecorded_trade_transactions(
    direction: str,
    my_trades_instrument_name: list,
    from_transaction_log_instrument: list,
) -> dict:
    """_summary_
    direction: "from_trans_log_to_my_trade" -> transaction log more update
    direction: "from_my_trade_to_trans_log" -> my_trade more update

    Returns:
        _type_: _description_
    """

    if direction == "from_trans_log_to_my_trade":

        from_transaction_log_instrument_trade_id = sorted(
            [o["trade_id"] for o in from_transaction_log_instrument]
        )

        if my_trades_instrument_name:
            my_trades_instrument_name_trade_id = sorted(
                [o["trade_id"] for o in my_trades_instrument_name]
            )

            if my_trades_instrument_name_trade_id:
                unrecorded_trade_id = str_mod.get_unique_elements(
                    from_transaction_log_instrument_trade_id,
                    my_trades_instrument_name_trade_id,
                )
        else:
            unrecorded_trade_id = from_transaction_log_instrument_trade_id

        log.debug(f"unrecorded_trade_from_transaction_log {unrecorded_trade_id}")

        return unrecorded_trade_id

    if direction == "from_my_trade_to_trans_log":

        from_transaction_log_instrument_trade_id = sorted(
            [o["trade_id"] for o in from_transaction_log_instrument]
        )

        if my_trades_instrument_name:
            my_trades_instrument_name_trade_id = [
                o["trade_id"] for o in my_trades_instrument_name
            ]

            if my_trades_instrument_name_trade_id:
                unrecorded_trade_id = str_mod.get_unique_elements(
                    my_trades_instrument_name_trade_id,
                    from_transaction_log_instrument_trade_id,
                )

        else:
            unrecorded_trade_id = []

        log.debug(f"unrecorded_trade_from_transaction_log {unrecorded_trade_id}")

        return unrecorded_trade_id

    if direction == "delivered":

        delivered_from_transaction_log_instrument = [
            o for o in from_transaction_log_instrument if o["type"] == "delivery"
        ]

        from_transaction_log_instrument_trade_id = sorted(
            str_mod.remove_redundant_elements(
                [o["timestamp"] for o in delivered_from_transaction_log_instrument]
            )
        )

        if my_trades_instrument_name:
            my_trades_instrument_name_trade_id = sorted(
                [o["timestamp"] for o in my_trades_instrument_name]
            )

            if my_trades_instrument_name_trade_id:
                unrecorded_time_stamp = str_mod.get_unique_elements(
                    from_transaction_log_instrument_trade_id,
                    my_trades_instrument_name_trade_id,
                )
        else:
            unrecorded_time_stamp = from_transaction_log_instrument_trade_id

        log.debug(
            f"unrecorded_time_stamp {unrecorded_time_stamp} {delivered_from_transaction_log_instrument}"
        )

        return [] if not unrecorded_time_stamp else max(unrecorded_time_stamp)


def get_transactions_with_closed_label(transactions_all: list) -> list:
    """ """
    transactions_with_labels = [o for o in transactions_all if o["label"] is not None]

    # log.error(f"transactions_with_labels {transactions_with_labels}")

    return (
        []
        if not transactions_with_labels
        else [o for o in transactions_with_labels if "closed" in o["label"]]
    )


def transactions_under_label_int(label_integer: int, transactions_all: list) -> str:
    """ """

    transactions = [o for o in transactions_all if str(label_integer) in o["label"]]

    return dict(
        closed_transactions=transactions,
        summing_closed_transaction=sum([o["amount"] for o in transactions]),
    )


def get_closed_open_transactions_under_same_label_int(
    transactions_all: list, label: str
) -> list:
    """ """
    label_integer = get_label_integer(label)

    return [o for o in transactions_all if str(label_integer) in o["label"]]


async def closing_orphan_order(
    label: str,
    label_integer: int,
    trade_table: str,
    where_filter: str,
    transactions_under_label_main: list,
) -> None:
    """ """

    closed_transaction_size = abs(
        [o["amount"] for o in transactions_under_label_main if label in o["label"]][0]
    )

    log.info(f" closed_transaction_size label {closed_transaction_size}")

    open_label_with_same_size_as_closed_label_int = [
        o
        for o in transactions_under_label_main
        if "open" in o["label"] and closed_transaction_size == abs(o["amount"])
    ]

    # log.warning(F"open_label_with_same_size_as_closed_label {open_label_with_same_size_as_closed_label_int}")

    if open_label_with_same_size_as_closed_label_int:

        open_label_with_the_same_label_int = [
            o
            for o in open_label_with_same_size_as_closed_label_int
            if label_integer in o["label"]
        ]

        log.warning(f"label_integer {label_integer}")

        if False and open_label_with_the_same_label_int:
            log.error(
                f"open_label_with_the_same_label_int {open_label_with_the_same_label_int}"
            )

            trade_id = closed_transaction[where_filter]

            # insert closed transaction to db for closed transaction
            await db_mgt.insert_tables("my_trades_closed_json", closed_transaction)

            # delete respective transaction form active db
            await db_mgt.deleting_row(
                trade_table,
                "databases/trading.sqlite3",
                where_filter,
                "=",
                trade_id,
            )


async def updating_status_closed_transactions(
    closed_transaction: dict,
    where_filter: str,
    trade_table: str,
) -> None:
    """ """
    trade_id = closed_transaction[where_filter]
    # trade_tabel = f"my_trades_all_{currency.lower()}.json"

    await db_mgt.update_status_data(
        trade_table,
        "is_open",
        where_filter,
        trade_id,
        0,
        "=",
    )


async def closing_one_to_one(
    transaction_closed_under_the_same_label_int: dict,
    where_filter: str,
    trade_table: str,
) -> None:
    """ """

    for transaction in transaction_closed_under_the_same_label_int:

        await updating_status_closed_transactions(
            transaction,
            where_filter,
            trade_table,
        )


async def closing_one_to_many_single_open_order(
    currency: str,
    open_label: dict,
    transaction_closed_under_the_same_label_int,
    where_filter: str,
    trade_table: str,
) -> None:

    open_label_size = abs(open_label["amount"])
    log.warning(f"open_label_size {open_label_size}")

    closed_label_with_same_size_as_open_label = [
        o
        for o in transaction_closed_under_the_same_label_int
        if "closed" in o["label"] and abs(o["amount"]) == open_label_size
    ]

    log.warning(
        f"closed_label_with_same_size_as_open_label{closed_label_with_same_size_as_open_label}"
    )

    if closed_label_with_same_size_as_open_label:

        closed_label_with_same_size_as_open_label_int = str(
            str_mod.extract_integers_aggregation_from_text(
                "trade_id", min, closed_label_with_same_size_as_open_label
            )
        )
        log.warning(
            f"closed_label_with_same_size_as_open_label_int {closed_label_with_same_size_as_open_label_int}"
        )

        closed_label_ready_to_close = (
            [
                o
                for o in closed_label_with_same_size_as_open_label
                if closed_label_with_same_size_as_open_label_int in o["trade_id"]
            ]
        )[0]

        log.debug(f"closed_label_ready_to_close{closed_label_ready_to_close}")

        # closed one of closing order
        await updating_status_closed_transactions(
            closed_label_ready_to_close, where_filter, trade_table
        )


async def closing_one_to_many(
    currency: str,
    open_label: str,
    transaction_closed_under_the_same_label_int: dict,
    where_filter: str,
    trade_table: str,
) -> None:
    """ """

    len_open_label_size = len(open_label)

    if len_open_label_size == 1:
        log.info(f"len_open_label_size 1 {len_open_label_size}")

        await closing_one_to_many_single_open_order(
            currency,
            open_label[0],
            transaction_closed_under_the_same_label_int,
            where_filter,
            trade_table,
        )

    else:
        for transaction in open_label:
            log.warning(f"len_open_label_size > 1 {len_open_label_size}")

            await closing_one_to_many_single_open_order(
                currency,
                transaction,
                transaction_closed_under_the_same_label_int,
                where_filter,
                trade_table,
            )


async def clean_up_closed_transactions(
    trade_table: str,
    transaction_all: list = None,
) -> None:
    """
    closed transactions: buy and sell in the same label id = 0. When flagged:
    1. remove them from db for open transactions/my_trades_all_json
    2. delete them from active trade db
    """

    # prepare basic parameters for table query

    # log.critical(f" {"clean_up_closed_transactions".upper()} {instrument_name} START")

    where_filter = f"trade_id"

    try:

        # filtered transactions with closing labels
        if transaction_all:

            transaction_with_closed_labels = get_transactions_with_closed_label(
                transaction_all
            )

            if transaction_with_closed_labels:

                labels_only = str_mod.remove_redundant_elements(
                    [o["label"] for o in transaction_with_closed_labels]
                )

                for label in labels_only:

                    transactions_under_label_main = get_label_main(
                        transaction_all, label
                    )

                    label_integer = get_label_integer(label)

                    closed_transactions_all = transactions_under_label_int(
                        label_integer, transactions_under_label_main
                    )

                    size_to_close = closed_transactions_all[
                        "summing_closed_transaction"
                    ]

                    transaction_closed_under_the_same_label_int = (
                        closed_transactions_all["closed_transactions"]
                    )

                    # log.error(f"closed_transactions_all {closed_transactions_all}")

                    if size_to_close == 0:

                        instrument_name_under_the_same_label_int = [
                            o["instrument_name"]
                            for o in transaction_closed_under_the_same_label_int
                        ]

                        for instrument_name in instrument_name_under_the_same_label_int:

                            instrument_transactions = [
                                o
                                for o in transaction_closed_under_the_same_label_int
                                if instrument_name in o["instrument_name"]
                            ]

                            sum_instrument_transactions = sum(
                                [o["amount"] for o in instrument_transactions]
                            )

                            if sum_instrument_transactions == 0:
                                # log.info(F" instrument_transactions {instrument_transactions}")

                                await closing_one_to_one(
                                    instrument_transactions,
                                    where_filter,
                                    trade_table,
                                )

                    if size_to_close != 0:

                        open_label = [
                            o
                            for o in transaction_closed_under_the_same_label_int
                            if "open" in o["label"]
                        ]

                        transactions = []
                        for open_transaction in open_label:

                            open_transaction_size = open_transaction["amount"]
                            open_transaction_instrument_name = open_transaction[
                                "instrument_name"
                            ]

                            closed_transaction_with_same_size_as_open_label = [
                                o
                                for o in transaction_closed_under_the_same_label_int
                                if "closed" in o["label"]
                                and abs(o["amount"]) == abs(open_transaction_size)
                            ]

                            if closed_transaction_with_same_size_as_open_label:

                                for (
                                    closed_transaction
                                ) in closed_transaction_with_same_size_as_open_label:

                                    closed_transaction_size = closed_transaction[
                                        "amount"
                                    ]
                                    closed_transaction_instrument_name = (
                                        closed_transaction["instrument_name"]
                                    )

                                    if (
                                        closed_transaction_instrument_name
                                        == open_transaction_instrument_name
                                        and closed_transaction_size
                                        + open_transaction_size
                                        == 0
                                    ):

                                        transactions.append(open_transaction)
                                        transactions.append(closed_transaction)

                                        log.critical(f"transactions {transactions}")
                                        log.warning(
                                            f"closed_transaction_instrument_name {closed_transaction_instrument_name}"
                                        )
                                        log.error(
                                            f"open_transaction_instrument_name {open_transaction_instrument_name}"
                                        )

                                        await closing_one_to_one(
                                            transactions,
                                            where_filter,
                                            trade_table,
                                        )

                                        if (
                                            closed_transaction_instrument_name
                                            == open_transaction_instrument_name
                                        ):

                                            break

    except Exception as error:

        error_handling.parse_error_message(error)



# -*- coding: utf-8 -*-

# built ins
import asyncio

# user defined formula
from streaming_helper.db_management import sqlite_management as db_mgt
from streaming_helper.utilities import error_handling


async def count_and_delete_ohlc_rows(
    database: str,
    table: str,
) -> None:

    try:
        rows_threshold = max_rows(table)

        if "supporting_items_json" in table or "account_summary_json" in table:
            where_filter = f"id"

        else:
            where_filter = f"tick"

        count_rows_query = db_mgt.querying_arithmetic_operator(
            where_filter, "COUNT", table
        )

        rows = await db_mgt.executing_query_with_return(count_rows_query)

        rows = (
            rows[0]["COUNT (tick)"] if where_filter == "tick" else rows[0]["COUNT (id)"]
        )

        if rows > rows_threshold:

            first_tick_query = db_mgt.querying_arithmetic_operator(
                where_filter, "MIN", table
            )

            first_tick_fr_sqlite = await db_mgt.executing_query_with_return(
                first_tick_query
            )

            if where_filter == "tick":
                first_tick = first_tick_fr_sqlite[0]["MIN (tick)"]

            if where_filter == "id":
                first_tick = first_tick_fr_sqlite[0]["MIN (id)"]

            await db_mgt.deleting_row(
                table,
                database,
                where_filter,
                "=",
                first_tick,
            )

    except Exception as error:
        error_handling.parse_error_message(error)


def max_rows(table) -> int:
    """ """
    if "market_analytics_json" in table:
        threshold = 10
    if "ohlc" in table:
        threshold = 10000
    if "supporting_items_json" in table:
        threshold = 200
    if "account_summary_json" in table:
        downloading_times = 2  # every 30 seconds
        currencies = 2
        instruments = 8
        threshold = (
            downloading_times * currencies * instruments
        ) * 120  # roughly = 2 hours

    return threshold


async def clean_up_databases(idle_time) -> None:
    """ """

    while True:

        tables = [
            "market_analytics_json",
            # "account_summary_json",
            "ohlc1_eth_perp_json",
            "ohlc1_btc_perp_json",
            "ohlc15_eth_perp_json",
            "ohlc15_btc_perp_json",
            # "ohlc30_eth_perp_json",
            "ohlc60_eth_perp_json",
            # "ohlc3_eth_perp_json",
            # "ohlc3_btc_perp_json",
            "ohlc5_eth_perp_json",
            "ohlc5_btc_perp_json",
        ]

        database: str = "databases/trading.sqlite3"

        for table in tables:

            await count_and_delete_ohlc_rows(database, table)

        await asyncio.sleep(idle_time)



# -*- coding: utf-8 -*-
"""_summary_"""
# built ins
import asyncio

# installed
from loguru import logger as log

# user defined formula
from streaming_helper.db_management import sqlite_management as db_mgt
from streaming_helper.utilities import string_modification as str_mod


def get_sub_account_size_per_instrument(
    instrument_name: str,
    sub_account: list,
) -> float:
    """ """

    sub_account_instrument = [
        o for o in sub_account if o["instrument_name"] == instrument_name
    ]

    sub_account_size_instrument = [o["size"] for o in sub_account_instrument]

    sub_account_size_instrument = (
        0 if sub_account_size_instrument == [] else sub_account_size_instrument[0]
    )

    return 0 if not sub_account_size_instrument else sub_account_size_instrument


def get_my_trades_size_per_instrument(
    instrument_name: str,
    my_trades_currency: list,
) -> float:
    """ """

    my_trades_instrument = (
        0
        if not my_trades_currency
        else [o for o in my_trades_currency if instrument_name in o["instrument_name"]]
    )

    sum_my_trades_instrument = (
        0
        if not my_trades_instrument
        else sum([o["amount"] for o in my_trades_instrument])
    )

    return 0 if not sum_my_trades_instrument else sum_my_trades_instrument


def get_transaction_log_position_per_instrument(
    instrument_name: str,
    from_transaction_log: list,
) -> float:
    """ """

    from_transaction_log_instrument = [
        o for o in from_transaction_log if o["instrument_name"] == instrument_name
    ]

    # timestamp could be double-> come from combo transaction. hence, trade_id is used to distinguish
    # other possibilities (instrument name beyond those in config):
    # from_transaction_log_instrument_example = [{'instrument_name': 'ETH_USDC', 'position': None, 'timestamp': 1730159747908, 'trade_id': 'ETH_USDC-3685325', 'user_seq': #1730159747919678, 'balance': 3987.0432}]

    try:

        last_time_stamp_log = (
            []
            if from_transaction_log_instrument == []
            else (max([(o["user_seq"]) for o in from_transaction_log_instrument]))
        )

        current_position_log = (
            0
            if not from_transaction_log_instrument
            else [
                o["position"]
                for o in from_transaction_log_instrument
                if last_time_stamp_log == o["user_seq"]
            ][0]
        )
        # just in case, trade id = None(because of settlement)
    except:

        examples_from_transaction_log_instrument = [
            {
                "instrument_name": "BTC-18OCT24",
                "position": 0,
                "timestamp": 1729238400029,
                "trade_id": None,
            },
            {
                "instrument_name": "BTC-18OCT24",
                "position": 0,
                "timestamp": 1729231480754,
                "trade_id": "321441856",
            },
            {
                "instrument_name": "BTC-18OCT24",
                "position": -100,
                "timestamp": 1728904931445,
                "trade_id": "320831413",
            },
        ]

        last_time_stamp_log = (
            []
            if from_transaction_log_instrument == []
            else str(
                max(
                    [
                        str_mod.extract_integers_from_text(o["trade_id"])
                        for o in from_transaction_log_instrument
                    ]
                )
            )
        )

        # log.error(f"last_time_stamp_log {last_time_stamp_log}")
        current_position_log = (
            0
            if not from_transaction_log_instrument
            else [
                o["position"]
                for o in from_transaction_log_instrument
                if str(last_time_stamp_log) in o["trade_id"]
            ][0]
        )

    return 0 if not current_position_log else current_position_log


def is_transaction_log_and_sub_account_size_reconciled_each_other(
    instrument_name: str,
    from_transaction_log: list,
    sub_account: list,
) -> bool:
    """ """

    current_position_log = get_transaction_log_position_per_instrument(
        instrument_name,
        from_transaction_log,
    )

    sub_account_size_instrument = get_sub_account_size_per_instrument(
        instrument_name, sub_account
    )

    reconciled = current_position_log == sub_account_size_instrument

    if not reconciled:
        log.critical(
            f"{instrument_name} reconciled {reconciled} sub_account_size_instrument {sub_account_size_instrument} current_position_log {current_position_log}"
        )

    return reconciled


def is_my_trades_active_archived_reconciled_each_other(
    instrument_name: str,
    my_trades_active: list,
    my_trades_archived: list,
) -> bool:
    """ """

    my_trades_active_size_instrument = get_my_trades_size_per_instrument(
        instrument_name,
        my_trades_active,
    )

    my_trades_archived_size_instrument = get_my_trades_size_per_instrument(
        instrument_name,
        my_trades_archived,
    )

    reconciled = my_trades_archived_size_instrument == my_trades_active_size_instrument

    log.warning(
        f"{instrument_name} reconciled {reconciled} my_trades_active_size_instrument {my_trades_active_size_instrument} my_trades_archived_size_instrument {my_trades_archived_size_instrument}"
    )

    if not reconciled:
        log.critical(
            f"{instrument_name} reconciled {reconciled} my_trades_active_size_instrument {my_trades_active_size_instrument} my_trades_archived_size_instrument {my_trades_archived_size_instrument}"
        )

    return reconciled


def is_my_trades_and_sub_account_size_reconciled_each_other(
    instrument_name: str,
    my_trades_currency: list,
    sub_account: list,
) -> bool:
    """ """

    my_trades_size_instrument = get_my_trades_size_per_instrument(
        instrument_name,
        my_trades_currency,
    )

    sub_account_size_instrument = get_sub_account_size_per_instrument(
        instrument_name,
        sub_account,
    )

    reconciled = my_trades_size_instrument == sub_account_size_instrument

    if not reconciled:
        log.critical(
            f"{instrument_name} reconciled {reconciled} sub_account_size_instrument {sub_account_size_instrument} my_trades_size_instrument {my_trades_size_instrument}"
        )

    return reconciled


async def my_trades_active_archived_not_reconciled_each_other(
    instrument_name: str,
    trade_db_table: str,
    archive_db_table: str,
    closed_db_table: str,
) -> None:

    column_trade: str = (
        "instrument_name",
        "data",
        "trade_id",
        "timestamp",
        "price",
        "amount",
    )

    my_trades_instrument_name_active = (
        await db_mgt.executing_query_based_on_currency_or_instrument_and_strategy(
            trade_db_table, instrument_name, "all", "all", column_trade
        )
    )

    my_trades_instrument_name_closed = (
        await db_mgt.executing_query_based_on_currency_or_instrument_and_strategy(
            closed_db_table, instrument_name, "all", "all", column_trade
        )
    )

    my_trades_instrument_name_archive = (
        await db_mgt.executing_query_based_on_currency_or_instrument_and_strategy(
            archive_db_table, instrument_name, "all", "all", column_trade
        )
    )

    my_trades_archive_instrument_sorted = str_mod.sorting_list(
        my_trades_instrument_name_archive, "timestamp", False
    )

    my_trades_currency_active_with_blanks = [
        o["id"]
        for o in my_trades_instrument_name_active
        if o["price"] is None or o["amount"] is None
    ]

    log.error(
        f"my_trades_currency_active_with_blanks {my_trades_currency_active_with_blanks}"
    )

    if my_trades_currency_active_with_blanks:
        for id in my_trades_currency_active_with_blanks:
            await db_mgt.deleting_row(
                "my_trades_all_json",
                "databases/trading.sqlite3",
                "id",
                "=",
                id,
            )

    my_trades_archive_instrument_data = [
        o["data"] for o in my_trades_archive_instrument_sorted
    ]

    if not my_trades_instrument_name_active and not my_trades_instrument_name_closed:

        for transaction in my_trades_archive_instrument_data:

            log.warning(
                f"my_trades_active_archived_not_reconciled_each_other {transaction} "
            )

            if transaction:
                await db_mgt.insert_tables(trade_db_table, transaction)
    else:

        from_sqlite_closed_trade_id = [
            o["trade_id"] for o in my_trades_instrument_name_closed
        ]

        from_sqlite_open_trade_id = [
            o["trade_id"] for o in my_trades_instrument_name_active
        ]

        from_exchange_trade_id = [
            o["trade_id"] for o in my_trades_instrument_name_archive
        ]

        combined_trade_closed_open = (
            from_sqlite_open_trade_id + from_sqlite_closed_trade_id
        )

        unrecorded_trade_id = str_mod.get_unique_elements(
            from_exchange_trade_id, combined_trade_closed_open
        )

        for trade_id in unrecorded_trade_id:

            transaction = [
                o
                for o in my_trades_instrument_name_archive
                if trade_id in o["trade_id"]
            ]

            log.debug(
                f"my_trades_active_archived_not_reconciled_each_other {transaction} "
            )

            await db_mgt.insert_tables(trade_db_table, transaction)


def is_size_sub_account_and_my_trades_reconciled(
    position_without_combo: list,
    sum_my_trades_currency_all: list,
    instrument_name: str,
) -> bool:
    """ """

    try:

        sub_account_size_instrument = (
            []
            if not position_without_combo
            else (
                [
                    (o["size"])
                    for o in position_without_combo
                    if instrument_name in o["instrument_name"]
                ]
            )
        )
        sub_account_size_instrument = (
            0 if sub_account_size_instrument == [] else sub_account_size_instrument[0]
        )

        my_trades_size_instrument = [
            o["amount"]
            for o in sum_my_trades_currency_all
            if instrument_name in o["instrument_name"]
        ]

        sum_my_trades_size_instrument = (
            0 if not my_trades_size_instrument else sum(my_trades_size_instrument)
        )

        if sub_account_size_instrument != sum_my_trades_size_instrument:
            log.critical(
                f"{instrument_name} sum_my_trades_size_instrument {sum_my_trades_size_instrument}  sub_account_size_instrument {sub_account_size_instrument}"
            )

        return sub_account_size_instrument == sum_my_trades_size_instrument

    except Exception as error:
        log.warning(error)


def check_whether_order_db_reconciled_each_other(
    sub_account: list, instrument_name: str, orders_currency: list
) -> None:
    """ """

    if sub_account:

        sub_account_orders = sub_account["open_orders"]

        sub_account_instrument = [
            o for o in sub_account_orders if o["instrument_name"] == instrument_name
        ]

        len_sub_account_instrument = (
            0
            if not sub_account_instrument
            else len([o["amount"] for o in sub_account_instrument])
        )

        orders_instrument = [
            o for o in orders_currency if instrument_name in o["instrument_name"]
        ]

        len_orders_instrument = (
            0
            if not orders_instrument
            else len([o["amount"] for o in orders_instrument])
        )

        result = len_orders_instrument == len_sub_account_instrument
        # log.debug (f"result {result} ")

        if not result:
            log.critical(
                f" {instrument_name} len_order equal {result} len_sub_account_instrument {len_sub_account_instrument} len_orders_instrument {len_orders_instrument}"
            )
            log.debug(sub_account_orders)
            log.warning(orders_currency)

        # comparing and return the result
        return result

    else:
        return False


def reading_from_pkl_data(end_point, currency, status: str = None) -> dict:
    """ """

    from utilities.pickling import read_data
    from utilities.system_tools import provide_path_for_file

    path: str = provide_path_for_file(end_point, currency, status)
    data = read_data(path)

    return data


async def reconciling_orders(
    modify_order_and_db: object,
    sub_account: list,
    orders_currency: list,
    direction: str,
    order_db_table: str,
) -> None:
    """
    direction:
            "from_sub_account_to_order_db"
            "from_order_db_to_sub_account"
    """

    try:

        sub_account_orders = sub_account["open_orders"]

        if orders_currency:

            if direction == "from_order_db_to_sub_account":
                orders_instrument_name = str_mod.remove_redundant_elements(
                    [o["instrument_name"] for o in orders_currency]
                )

            if direction == "from_sub_account_to_order_db":

                orders_instrument_name = str_mod.remove_redundant_elements(
                    [o["instrument_name"] for o in sub_account_orders]
                )

            if orders_instrument_name:

                for instrument_name in orders_instrument_name:

                    currency = str_mod.extract_currency_from_text(instrument_name)

                    len_order_is_reconciled_each_other = (
                        check_whether_order_db_reconciled_each_other(
                            sub_account, instrument_name, orders_currency
                        )
                    )

                    if not len_order_is_reconciled_each_other:

                        sub_account_instrument_name = [
                            o
                            for o in sub_account_orders
                            if instrument_name in o["instrument_name"]
                        ]

                        where_filter = f"instrument_name"

                        await db_mgt.deleting_row(
                            "orders_all_json",
                            "databases/trading.sqlite3",
                            where_filter,
                            "=",
                            instrument_name,
                        )

                        for order in sub_account_instrument_name:

                            await db_mgt.insert_tables(order_db_table, order)

                        await modify_order_and_db.resupply_sub_accountdb(currency)

    except Exception as error:
        log.warning(error)



# -*- coding: utf-8 -*-

# built ins
import asyncio

# installed
import orjson
from loguru import logger as log

# user defined formula
from streaming_helper.db_management.redis_client import publishing_result
from streaming_helper.db_management.sqlite_management import (
    executing_query_with_return,
    insert_tables,
    querying_arithmetic_operator,
    update_status_data,
)
from streaming_helper.restful_api.deribit import end_point_params_template as end_point
from streaming_helper.restful_api import connector
from streaming_helper.utilities import error_handling, string_modification as str_mod


async def last_tick_fr_sqlite(last_tick_query_ohlc1: str) -> int:
    """ """
    last_tick = await executing_query_with_return(last_tick_query_ohlc1)

    return last_tick[0]["MAX (tick)"]


async def updating_ohlc(
    client_redis: object,
    redis_channels: list,
) -> None:
    """ """

    try:

        # connecting to redis pubsub
        pubsub: object = client_redis.pubsub()

        chart_channel: str = redis_channels["chart_update"]
        chart_low_high_tick_channel: str = redis_channels["chart_low_high_tick"]

        # prepare channels placeholders
        channels = [
            chart_channel,
        ]

        # subscribe to channels
        [await pubsub.subscribe(o) for o in channels]

        WHERE_FILTER_TICK: str = "tick"

        is_updated = True

        start_timestamp = 0

        basic_https_connection_url = end_point.basic_https()

        endpoint_tradingview = end_point.endpoint_tradingview()

        while is_updated:

            try:

                message_byte = await pubsub.get_message()

                #                log.info(f"message_byte {message_byte}")

                if message_byte and message_byte["type"] == "message":

                    message_byte_data = orjson.loads(message_byte["data"])

                    message_channel = message_byte["channel"]

                    if chart_channel in message_channel:

                        data = message_byte_data["data"]

                        instrument_name = message_byte_data["instrument_name"]

                        currency = message_byte_data["currency"]

                        resolution = message_byte_data["resolution"]

                        end_timestamp = data["tick"]

                        table_ohlc = f"ohlc{resolution}_{currency.lower()}_perp_json"

                        last_tick_query_ohlc_resolution: str = (
                            querying_arithmetic_operator(
                                WHERE_FILTER_TICK,
                                "MAX",
                                table_ohlc,
                            )
                        )

                        # need cached
                        start_timestamp: int = await last_tick_fr_sqlite(
                            last_tick_query_ohlc_resolution
                        )

                        delta_time = end_timestamp - start_timestamp

                        pub_message = dict(
                            instrument_name=instrument_name,
                            currency=currency,
                            resolution=resolution,
                        )

                        if delta_time == 0:

                            # refilling current ohlc table with updated data
                            await update_status_data(
                                table_ohlc,
                                "data",
                                end_timestamp,
                                WHERE_FILTER_TICK,
                                data,
                                "is",
                            )

                            if resolution != 1:

                                table_ohlc = (
                                    f"ohlc{resolution}_{currency.lower()}_perp_json"
                                )

                                ohlc_query = f"SELECT data FROM {table_ohlc} WHERE tick = {end_timestamp}"

                                result_from_sqlite = await executing_query_with_return(
                                    ohlc_query
                                )

                                high_from_ws = data["high"]
                                low_from_ws = data["low"]

                                ohlc_from_sqlite = str_mod.remove_apostrophes_from_json(
                                    o["data"] for o in result_from_sqlite
                                )[0]

                                high_from_db = ohlc_from_sqlite["high"]
                                low_from_db = ohlc_from_sqlite["low"]

                                if (
                                    high_from_ws > high_from_db
                                    or low_from_ws < low_from_db
                                ):

                                    await publishing_result(
                                        client_redis,
                                        chart_low_high_tick_channel,
                                        pub_message,
                                    )

                        else:

                            endpoint_ohlc = end_point.get_ohlc_end_point(
                                endpoint_tradingview,
                                instrument_name,
                                resolution,
                                start_timestamp,
                                end_timestamp,
                                False,
                            )

                            # catch up data through FIX
                            ohlc = await connector.get_connected(
                                basic_https_connection_url,
                                endpoint_ohlc,
                            )

                            result_all = str_mod.transform_nested_dict_to_list_ohlc(
                                ohlc
                            )

                            await publishing_result(
                                client_redis,
                                chart_low_high_tick_channel,
                                pub_message,
                            )

                            for result in result_all:

                                await insert_tables(
                                    table_ohlc,
                                    result,
                                )

            except Exception as error:

                await error_handling.parse_error_message_with_redis(
                    client_redis,
                    error,
                )

                continue

            finally:
                await asyncio.sleep(0.001)

    except Exception as error:

        await error_handling.parse_error_message_with_redis(
            client_redis,
            error,
        )


async def inserting_open_interest(
    currency,
    WHERE_FILTER_TICK,
    TABLE_OHLC1,
    data_orders,
) -> None:
    """ """
    try:

        if (
            currency_inline_with_database_address(currency, TABLE_OHLC1)
            and "open_interest" in data_orders
        ):

            open_interest = data_orders["open_interest"]

            last_tick_query_ohlc1: str = querying_arithmetic_operator(
                "tick", "MAX", TABLE_OHLC1
            )

            last_tick1_fr_sqlite: int = await last_tick_fr_sqlite(last_tick_query_ohlc1)

            await update_status_data(
                TABLE_OHLC1,
                "open_interest",
                last_tick1_fr_sqlite,
                WHERE_FILTER_TICK,
                open_interest,
                "is",
            )

    except Exception as error:

        await error_handling.parse_error_message_with_redis(
            client_redis,
            error,
        )


def currency_inline_with_database_address(
    currency: str,
    database_address: str,
) -> bool:

    return currency.lower() in str(database_address)



# -*- coding: utf-8 -*-

import asyncio

from dataclassy import dataclass

# user defined formula
from streaming_helper.utilities import (
    time_modification as time_mod,
    string_modification as str_mod,
)
from streaming_helper.restful_api import connector


def get_basic_https() -> str:
    return f"https://www.deribit.com/api/v2/"


def get_currencies_end_point() -> str:
    return f"public/get_currencies?"


def get_server_time_end_point() -> str:
    return f"public/get_time?"


def get_instruments_end_point(currency) -> str:
    return f"public/get_instruments?currency={currency.upper()}"


async def get_instruments(currency) -> str:

    result = await connector.get_connected(
        get_basic_https(),
        get_instruments_end_point(currency),
    )

    return result["result"]


def get_tickers_end_point(instrument_name: str) -> str:

    return f"public/ticker?instrument_name={instrument_name}"


async def get_ticker(instrument_name) -> str:

    result = await connector.get_connected(
        get_basic_https(),
        get_tickers_end_point(instrument_name),
    )

    return result["result"]


def get_tradingview_chart_data_end_point() -> str:
    return f"get_tradingview_chart_data?"


def get_ohlc_end_point(
    endpoint_tradingview: str,
    instrument_name: str,
    resolution: int,
    qty_or_start_time_stamp: int,
    provided_end_timestamp: int = None,
    qty_as_start_time_stamp: bool = False,
) -> str:

    now_unix = time_mod.get_now_unix_time()

    # start timestamp is provided
    start_timestamp = qty_or_start_time_stamp

    # recalculate start timestamp using qty as basis point
    if qty_as_start_time_stamp:
        start_timestamp = now_unix - (60000 * resolution) * qty_as_start_time_stamp

    if provided_end_timestamp:
        end_timestamp = provided_end_timestamp
    else:
        end_timestamp = now_unix

    return f"{endpoint_tradingview}end_timestamp={end_timestamp}&instrument_name={instrument_name}&resolution={resolution}&start_timestamp={start_timestamp}"


def get_json_payload(
    endpoint: str,
    params: dict,
) -> dict:

    id = id_numbering(
        endpoint,
        endpoint,
    )

    return {
        "jsonrpc": "2.0",
        "id": id,
        "method": f"{endpoint}",
        "params": params,
    }


def get_open_orders_end_point() -> str:
    return f"private/get_open_orders"


def get_open_orders_params(
    kind: str,
    type: str,
) -> dict:
    return {
        "kind": kind,
        "type": type,
    }


def get_subaccounts_end_point() -> str:
    return f"private/get_subaccounts"


def get_subaccounts_params(
    with_portfolio: bool = True,
) -> dict:
    return {
        "with_portfolio": with_portfolio,
    }


def get_subaccounts_details_end_point() -> str:
    return f"private/get_subaccounts_details"


def get_subaccounts_details_params(
    currency: str,
    with_open_orders: bool = True,
) -> dict:
    return {
        "currency": currency,
        "with_open_orders": with_open_orders,
    }


def get_transaction_log_end_point() -> str:
    return f"private/get_transaction_log"


def get_transaction_log_params(
    currency: str,
    start_timestamp: int,
    count: int = 1000,
    query: str = "trade",
) -> dict:

    now_unix = time_mod.get_now_unix_time()

    return {
        "count": count,
        "currency": currency,
        "end_timestamp": now_unix,
        "query": query,
        "start_timestamp": start_timestamp,
    }


def send_orders_end_point(side: dict) -> str:

    return f"private/{side}"


def send_orders_params(
    side: str,
    instrument,
    amount,
    label: str = None,
    price: float = None,
    type: str = "limit",
    otoco_config: list = None,
    linked_order_type: str = None,
    trigger_price: float = None,
    trigger: str = "last_price",
    time_in_force: str = "fill_or_kill",
    reduce_only: bool = False,
    post_only: bool = True,
    reject_post_only: bool = False,
) -> str:

    params = {}

    params.update({"instrument_name": instrument})
    params.update({"amount": amount})
    params.update({"label": label})
    params.update({"instrument_name": instrument})
    params.update({"type": type})

    if trigger_price is not None:

        params.update({"trigger": trigger})
        params.update({"trigger_price": trigger_price})
        params.update({"reduce_only": reduce_only})

    if "market" not in type:
        params.update({"price": price})
        params.update({"post_only": post_only})
        params.update({"reject_post_only": reject_post_only})

    if otoco_config:
        params.update({"otoco_config": otoco_config})

        if linked_order_type is not None:
            params.update({"linked_order_type": linked_order_type})
        else:
            params.update({"linked_order_type": "one_triggers_other"})

        params.update({"trigger_fill_condition": "incremental"})

    return params


def send_limit_order_params(
    params: dict,
) -> str:
    """ """

    # basic params
    side = params["side"]
    instrument = params["instrument_name"]
    label_numbered = params["label"]
    size = params["size"]
    type = params["type"]
    limit_prc = params["entry_price"]

    try:
        otoco_config = params["otoco_config"]
    except:
        otoco_config = None

    try:
        linked_order_type = params["linked_order_type"]

    except:
        linked_order_type = None

    order_result = None

    if side != None:

        if type == "limit":  # limit has various state

            order_result = send_orders_params(
                side,
                instrument,
                size,
                label_numbered,
                limit_prc,
                type,
                otoco_config,
            )

        else:

            trigger_price = params["trigger_price"]
            trigger = params["trigger"]

            order_result = send_orders_params(
                side,
                instrument,
                size,
                label_numbered,
                limit_prc,
                type,
                otoco_config,
                linked_order_type,
                trigger_price,
                trigger,
            )

    return order_result


def cancel_all_orders_end_point() -> str:
    return f"private/cancel_all"


def cancel_order_end_point() -> str:
    return f"private/cancel"


def get_cancel_order_params(
    order_id: str,
) -> dict:

    return {"order_id": order_id}


def get_user_trades_by_currency_end_point() -> str:
    return f"private/get_user_trades_by_currency"


def get_user_trades_by_instrument_and_time_end_point() -> str:
    return f"private/get_user_trades_by_instrument_and_time"


def get_user_trades_by_instrument_and_time_params(
    instrument_name: str,
    start_timestamp: int,
    historical: bool = True,
    count: int = 1000,
) -> dict:

    now_unix = time_mod.get_now_unix_time()

    return {
        "count": count,
        "end_timestamp": now_unix,
        "instrument_name": instrument_name,
        "historical": historical,
        "start_timestamp": start_timestamp,
    }


def simulate_portfolio_end_point() -> str:
    return f"private/simulate_portfolio"


def simulate_portfolio_params(
    instrument_name: str,
    position: int,
    add_positions: bool = True,
) -> dict:

    currency = str_mod.extract_currency_from_text(instrument_name)

    return {
        "currency": currency,
        "add_positions": add_positions,
        "simulated_positions": {f""" "{instrument_name}" """: {position}},
    }


@dataclass(unsafe_hash=True, slots=True)
class SendApiRequest:
    """ """

    client_id: str
    client_secret: str

    async def simulate_portfolio(
        self,
        instrument_name: str,
        position: int,
        add_positions: bool = True,
    ) -> None:
        """
        result_example = {
            "jsonrpc": "2.0",
            "id": 5,
            "result": {
                "total_delta_total_usd": 5.30812019,
                "options_theta_map": {},
                "projected_maintenance_margin": 0.000727,
                "maintenance_margin": 0.000727,
                "initial_margin": 0.000979,
                "futures_session_rpl": 0,
                "spot_reserve": 0,
                "balance": 0.000567,
                "available_subaccount_transfer_funds": 0,
                "options_theta": 0,
                "margin_balance": 0.002644,
                "cross_collateral_enabled": True,
                "total_maintenance_margin_usd": 1.334470538932,
                "options_session_rpl": 0,
                "projected_delta_total": 0.002742,
                "locked_balance": 0,
                "options_vega": 0,
                "portfolio_margining_enabled": True,
                "options_value": 0,
                "equity": 0.000574,
                "futures_session_upl": 0.000006,
                "futures_pl": -0.000068,
                "session_upl": 0.000006,
                "total_margin_balance_usd": 4.852886723,
                "delta_total": 0.002742,
                "options_vega_map": {},
                "total_pl": -0.000068,
                "margin_model": "cross_pm",
                "options_session_upl": 0,
                "options_delta": 0,
                "fee_balance": 0,
                "session_rpl": 0,
                "currency": "ETH",
                "available_funds": 0.001664,
                "options_pl": 0,
                "options_gamma": 0,
                "projected_initial_margin": 0.000979,
                "available_withdrawal_funds": 0.000567,
                "total_initial_margin_usd": 1.797687367,
                "total_equity_usd": 4.852886723,
                "delta_total_map": {
                "eth_usd": 0.002742375
                },
                "additional_reserve": 0,
                "options_gamma_map": {}
                },
            "usIn": 1746618638373417,
            "usOut": 1746618638377054,
            "usDiff": 3637,
            "testnet": False
            }

        """

        portfolio_simulation = await connector.get_connected(
            get_basic_https(),
            simulate_portfolio_end_point(),
            self.client_id,
            self.client_secret,
            simulate_portfolio_params(
                instrument_name,
                position,
                add_positions,
            ),
        )

        return portfolio_simulation["result"]

    async def get_cancel_order_byOrderId(
        self,
        order_id: str,
    ) -> None:
        """ """

        cancel_order = await connector.get_connected(
            get_basic_https(),
            cancel_order_end_point(),
            self.client_id,
            self.client_secret,
            get_cancel_order_params(
                order_id,
            ),
        )

        return cancel_order["result"]

    async def send_limit_order(
        self,
        params: dict,
    ) -> None:
        """ """

        # basic params
        side = params["side"]
        instrument = params["instrument_name"]
        label_numbered = params["label"]
        size = params["size"]
        type = params["type"]
        limit_prc = params["entry_price"]

        try:
            otoco_config = params["otoco_config"]
        except:
            otoco_config = None

        try:
            linked_order_type = params["linked_order_type"]

        except:
            linked_order_type = None

        order_result = None

        if side != None:

            if type == "limit":  # limit has various state

                order_result = await self.send_order(
                    side,
                    instrument,
                    size,
                    label_numbered,
                    limit_prc,
                    type,
                    otoco_config,
                )

            else:

                trigger_price = params["trigger_price"]
                trigger = params["trigger"]

                order_result = await self.send_order(
                    side,
                    instrument,
                    size,
                    label_numbered,
                    limit_prc,
                    type,
                    otoco_config,
                    linked_order_type,
                    trigger_price,
                    trigger,
                )

        # log.warning(f"order_result {order_result}")

        if order_result != None and (
            "error" in order_result or "message" in order_result
        ):

            error = order_result["error"]
            message = error["message"]

            try:
                data = error["data"]
            except:
                data = message

        return order_result

    async def send_order(
        self,
        side: str,
        instrument,
        amount,
        label: str = None,
        price: float = None,
        type: str = "limit",
        otoco_config: list = None,
        linked_order_type: str = None,
        trigger_price: float = None,
        trigger: str = "last_price",
        time_in_force: str = "fill_or_kill",
        reduce_only: bool = False,
        post_only: bool = True,
        reject_post_only: bool = False,
    ) -> None:

        params = {}

        params.update({"instrument_name": instrument})
        params.update({"amount": amount})
        params.update({"label": label})
        params.update({"instrument_name": instrument})
        params.update({"type": type})

        if trigger_price is not None:

            params.update({"trigger": trigger})
            params.update({"trigger_price": trigger_price})
            params.update({"reduce_only": reduce_only})

        if "market" not in type:
            params.update({"price": price})
            params.update({"post_only": post_only})
            params.update({"reject_post_only": reject_post_only})

        if otoco_config:
            params.update({"otoco_config": otoco_config})

            if linked_order_type is not None:
                params.update({"linked_order_type": linked_order_type})
            else:
                params.update({"linked_order_type": "one_triggers_other"})

            params.update({"trigger_fill_condition": "incremental"})

        result = None

        if side is not None:

            result = await connector.get_connected(
                get_basic_https(),
                send_orders_end_point(side),
                self.client_id,
                self.client_secret,
                send_orders_params(
                    side,
                    instrument,
                    amount,
                    label,
                    price,
                    type,
                    otoco_config,
                    linked_order_type,
                    trigger_price,
                    trigger,
                    time_in_force,
                    reduce_only,
                    post_only,
                    reject_post_only,
                ),
            )

        return result

    async def get_subaccounts(
        self,
        with_portfolio: bool = True,
    ) -> dict:

        sub_account = await connector.get_connected(
            get_basic_https(),
            get_subaccounts_end_point(),
            self.client_id,
            self.client_secret,
            get_subaccounts_params(with_portfolio),
        )

        return sub_account["result"]

    async def get_subaccounts_details(
        self,
        currency: str,
        with_open_orders: bool = True,
    ) -> dict:

        sub_account = await connector.get_connected(
            get_basic_https(),
            get_subaccounts_details_end_point(),
            self.client_id,
            self.client_secret,
            get_subaccounts_details_params(
                currency,
                with_open_orders,
            ),
        )

        return sub_account["result"]

    async def get_transaction_log(
        self,
        currency: str,
        start_timestamp: int,
        count: int = 1000,
        query: str = "trade",
    ) -> list:

        result_transaction_log = await connector.get_connected(
            get_basic_https(),
            get_transaction_log_end_point(),
            self.client_id,
            self.client_secret,
            get_transaction_log_params(
                currency,
                start_timestamp,
                count,
                query,
            ),
        )

        result = result_transaction_log["result"]

        return [] if not result else result["logs"]

    async def get_open_orders(
        self,
        kind: str,
        type: str,
    ) -> list:

        result_open_orders = await connector.get_connected(
            get_basic_https(),
            get_open_orders_end_point(),
            self.client_id,
            self.client_secret,
            get_open_orders_params(
                kind,
                type,
            ),
        )

        return result_open_orders["result"]

    async def get_user_trades_by_instrument_and_time(
        self,
        instrument_name: str,
        start_timestamp: int,
        historical: True,
        count: 1000,
    ) -> list:
        """
        result_example = {
            'trades': [
                {
                    'label': 'customLong-open-1746435478513',
                    'timestamp': 1746435505432,
                    'state': 'filled',
                    'price': 1827.75,
                    'amount': 1.0,
                    'direction': 'buy',
                    'index_price': 1827.94,
                    'profit_loss': 0.0,
                    'instrument_name': 'ETH-PERPETUAL',
                    'trade_seq': 179290420,
                    'api': True,
                    'mark_price': 1827.67,
                    'order_id': 'ETH-67544635984',
                    'matching_id': None,
                    'tick_direction': 3,
                    'fee': 0.0,
                    'mmp': False,
                    'self_trade': False,
                    'post_only': True,
                    'reduce_only': False,
                    'contracts': 1.0,
                    'trade_id': 'ETH-247401821',
                    'fee_currency': 'ETH',
                    'order_type': 'limit',
                    'risk_reducing': False,
                    'liquidity': 'M'
                    }
                ],
            'has_more': False
            }

        """

        result_trades = await connector.get_connected(
            get_basic_https(),
            get_user_trades_by_instrument_and_time_end_point(),
            self.client_id,
            self.client_secret,
            get_user_trades_by_instrument_and_time_params(
                instrument_name,
                start_timestamp,
                historical,
                count,
            ),
        )

        return result_trades["result"]["trades"]


def get_user_trades_by_currency_params(
    currency: str,
    kind: str = "any",
    count: int = 1000,
) -> dict:
    return {
        "currency": currency,
        "kind": kind,
        "count": count,
    }


def get_api_end_point(
    endpoint: str,
    parameters: dict = None,
) -> dict:
    """
    used in data receiver deribit

    """

    private_endpoint = f"private/{endpoint}"

    params = {}
    params.update({"jsonrpc": "2.0"})
    params.update({"method": private_endpoint})
    if endpoint == "get_subaccounts":
        params.update({"params": {"with_portfolio": True}})

    if endpoint == "get_open_orders":
        end_point_params = dict(kind=parameters["kind"], type=parameters["type"])

        params.update({"params": end_point_params})

    return params


def id_numbering(
    operation: str,
    ws_channel: str,
) -> str:
    """

    id convention

    method
    subscription    3
    get             4

    auth
    public	        1
    private	        2

    instruments
    all             0
    btc             1
    eth             2

    subscription
    --------------  method      auth    seq    inst
    portfolio	        3	    1	    01
    user_order	        3	    1	    02
    my_trade	        3	    1	    03
    order_book	        3	    2	    04
    trade	            3	    1	    05
    index	            3	    1	    06
    announcement	    3	    1	    07

    get
    --------------
    currencies	        4	    2	    01
    instruments	        4	    2	    02
    positions	        4	    1	    03

    """
    id_auth = 1
    if "user" in ws_channel:
        id_auth = 9

    id_method = 0
    if "subscribe" in operation:
        id_method = 3
    if "get" in operation:
        id_method = 4
    id_item = 0
    if "book" in ws_channel:
        id_item = 1
    if "user" in ws_channel:
        id_item = 2
    if "chart" in ws_channel:
        id_item = 3
    if "index" in ws_channel:
        id_item = 4
    if "order" in ws_channel:
        id_item = 5
    if "position" in ws_channel:
        id_item = 6
    if "simulation" in ws_channel:
        id_item = 7
    id_instrument = 0
    if "BTC" or "btc" in ws_channel:
        id_instrument = 1
    if "ETH" or "eth" in ws_channel:
        id_instrument = 2
    return int(f"{id_auth}{id_method}{id_item}{id_instrument}")
